/**************************************************************************
**       Title: test program for SimpleRGBImage
**    $RCSfile: testSimpleRGBImage.cc,v $
**   $Revision: 1.4 $$Name:  $
**       $Date: 2002/11/13 11:25:37 $
**   Copyright: GPL $Author: pigorsch $
** Description:
**
**    
**
**-------------------------------------------------------------------------
**
**  $Log: testSimpleRGBImage.cc,v $
**  Revision 1.4  2002/11/13 11:25:37  pigorsch
**  - added more tests to testRead
**
**  Revision 1.3  2002/11/05 16:06:02  pigorsch
**  - reduced test image size
**
**  Revision 1.2  2002/11/04 14:25:23  pigorsch
**  *** empty log message ***
**
**  Revision 1.1  2002/11/02 12:07:11  pigorsch
**  - initial revision
**
**
**
**************************************************************************/

#include <iostream>
#include <fstream>

#include "MyError.hh"
#include "SimpleRGBImage.hh"

/*======================================================================*/
/*! 
 *   Check if the default constructor works.
 */
/*======================================================================*/
static bool testEmptyContructor()
{
  std::cout << "testEmptyContructor\n";
  
  SimpleRGBImage image;
  
  if (image.width()!=0 || image.height()!=0 || 
      image.size()!=0 ||
      image.begin()!=image.end())
  {
    std::cout << "[properties] failed\n";
    return false;
  }

  std::cerr << "ok\n";
  return true;
}

/*======================================================================*/
/*! 
 *   Check if the copy constructor works.
 */
/*======================================================================*/
static bool testCopyConstructor()
{
  std::cerr << "testCopyConstructor\n";
  
  // create blue image
  SimpleRGBImage image1(5, 5);
  for (RGBColor* p=image1.begin();
       p!=image1.end();
       ++p)
  {
    *p=RGBColor::Blue;
  }
  
  // copy image
  SimpleRGBImage image2(image1);

  // check if both images have equal properties
  if (image1.width()!=image2.width() ||
      image1.height()!=image2.height() ||
      image1.size()!=image2.size())
  {
    std::cerr << "[properties] failed\n";
    return false;
  }
  
  // check if pointers are different
  if (image1.begin()==image2.begin() ||
      image1.end()==image2.end())
  {
    std::cerr << "[pointers] failed\n";
    return false;
  }

  // check if copying also copied pixels
  for (const RGBColor* p=image2.begin();
       p!=image2.end();
       ++p)
  {
    if (*p!=RGBColor::Blue)
    {
      std::cerr << "[pixels] failed\n";
      return false;
    }
  }
  
  // change one pixel in new image
  image2(3, 3)=RGBColor::Red;
  
  // check if pixel changed not in original image but in new one
  if (image1(3, 3)!=RGBColor::Blue || image2(3, 3)!=RGBColor::Red)
  {
    std::cerr << "[allocation] failed\n";
    return false;
  }
  
  std::cerr << "ok\n";
  return true;
}  

/*======================================================================*/
/*! 
 *   Check if the size constructor works.
 */
/*======================================================================*/
static bool testSizeConstructor()
{
  std::cout << "testSizeConstructor\n";
  
  // create image of arbitrary size
  SimpleRGBImage image(42, 17);
  // check if properties are correct
  if (image.width()!=42 || image.height()!=17 || image.size()!=42*17 ||
      image.begin()+image.size()!=image.end())
  {
    std::cerr << "[properties] failed\n";
    return true;
  }
  
  std::cerr << "ok\n";
  return true;
}

/*======================================================================*/
/*! 
 *   Check if resize() works.
 */
/*======================================================================*/
static bool testResize()
{
  std::cout << "testResize\n";
  
  // create image
  SimpleRGBImage image(42, 17);
  // resize image
  image.resize(117, 5);
  // check if properties are correct
  if (image.width()!=117 || image.height()!=5 || image.size()!=117*5 ||
      image.begin()+image.size()!=image.end())
  {
    std::cerr << "[properties] failed\n";
    return true;
  }
  
  // set color of arbitrary pixel
  image(116, 4)=RGBColor::Blue;
  // try to access that pixel by op[]
  if (image[117*5-1]!=RGBColor::Blue)
  {
    std::cerr << "[pixel access] failed\n";
    return true;
  }
  
  // try to make image really huge
  try
  {
    image.resize(1000000, 1000000);
  }
  catch (const MyError& e) // correct exception thrown
  {
  }
  catch (...) // probably bad_alloc exception
  {
    std::cerr << "[oversize] failed\n";
    return false;
  }
  
  std::cerr << "ok\n";
  return true;
}
 
/*======================================================================*/
/*! 
 *   Check if read() works.
 */
/*======================================================================*/
static bool testRead()
{
  std::cerr << "testRead\n";

  SimpleRGBImage image;

  std::ifstream file1("16x16_red_gradient.ppm");
  image.read(file1);

  // check properties
  if (image.width()!=16 || image.height()!=16 ||
      image[0]!=RGBColor::Red || image[image.size()-1]!=RGBColor::White)
  {
    std::cerr << "[load correct] failed\n";
    return false;
  }

  // read file with many comments
  bool ok=true;
  std::ifstream file2("16x16_red_gradient.ppm");
  try
  {
    image.read(file2);
  }
  catch (const MyError& e)
  {
    ok=false;
  }
  catch (...)
  {
    ok=false;
  }
  
  if (!ok)
  {
    std::cerr << "[load comments] failed\n";
    return false;
  }
  
  
  // read file with invalid header
  ok=false;
  std::ifstream file3("badheader.ppm");
  try
  {
    image.read(file3);
  }
  catch (const MyError& e)
  {
    ok=true;
  }
  catch (...)
  {
    ok=false;
  }
  
  if (!ok)
  {
    std::cerr << "[load badheader] failed\n";
    return false;
  }
  
  
  // read truncated
  ok=false;
  std::ifstream file4("truncated.ppm");
  try
  {
    image.read(file4);
  }
  catch (const MyError& e)
  {
    ok=true;
  }
  catch (...)
  {
    ok=false;
  }
  
  if (!ok)
  {
    std::cerr << "[load truncated] failed\n";
    return false;
  }
  

  // read oversize file
  ok=false;
  std::ifstream file5("oversize.ppm");
  try
  {
    image.read(file5);
  }
  catch (const MyError& e)
  {
    ok=true;
  }
  catch (...)
  {
    ok=false;
  }
  
  if (!ok)
  {
    std::cerr << "[load oversize] failed\n";
    return false;
  }
  


  std::cerr << "ok\n";
  return true; 
}

/*======================================================================*/
/*! 
 *   Check if write() works.
 */
/*======================================================================*/
static bool testWrite()
{
  std::cerr << "testWrite\n";
  
  // create image
  SimpleRGBImage image(16, 16);
  // set pixel colors
  for (unsigned int y=0; y<image.height(); ++y)
  {
    for (unsigned int x=0; x<image.width(); ++x)
    {
      image(x,y).set(x, 0, y);
    }
  }
  // write image
  std::ofstream file("out.ppm");
  image.write(file);
  file.flush();

  std::ifstream filein("out.ppm");
  SimpleRGBImage image2(filein);

  if (image.size()!=image2.size() ||
      image.width()!=image2.width() ||
      image.height()!=image2.height())
  {
    std::cerr << "[properties failed]\n";
    return false;
  }
  
  for (unsigned int y=0; y<image.height(); ++y)
  {
    for (unsigned int x=0; x<image.width(); ++x)
    {
      if (image(x,y)!=image2(x,y))
      {
        std::cerr << "[pixels failed]\n";
        return false;
      }
    }
  }

  std::cerr << "ok\n";
  return true;
}

/*======================================================================*/
/*! 
 *   Check if operator=() works, see testCopyConstructor()
 */
/*======================================================================*/
static bool testAssignmentOperator()
{
  std::cerr << "testAssignmentOperator\n";
 
  SimpleRGBImage image1(5, 5);
  for (RGBColor* p=image1.begin();
       p!=image1.end();
       ++p)
  {
    *p=RGBColor::Blue;
  }
  
  SimpleRGBImage image2(3, 3);
  image2=image1;

  if (image1.width()!=image2.width() ||
      image1.height()!=image2.height() ||
      image1.size()!=image2.size())
  {
    std::cerr << "[properties] failed\n";
    return false;
  }
  
  if (image1.begin()==image2.begin() ||
      image1.end()==image2.end())
  {
    std::cerr << "[pointers] failed\n";
    return false;
  }

  for (const RGBColor* p=image2.begin();
       p!=image2.end();
       ++p)
  {
    if (*p!=RGBColor::Blue)
    {
      std::cerr << "[pixels] failed\n";
      return false;
    }
  }

  for (RGBColor* p=image2.begin();
       p!=image2.end();
       ++p)
  {
    *p=RGBColor::Red;
  }


  if (image1(3, 3)!=RGBColor::Blue || image2(3, 3)!=RGBColor::Red)
  {
    std::cerr << "[allocation] failed\n";
    return false;
  }
  
  std::cerr << "ok\n";
  return true;
}

int main()
{
  try
  {    
    testEmptyContructor();
    testCopyConstructor();
    testSizeConstructor();
    testResize();  
    testRead();
    testWrite();
    testAssignmentOperator();
  }
  catch (const MyError& e)
  {
    std::cerr << "MyError caught: " << e.msg() << std::endl;
  }
  catch (...)
  {
    std::cerr << "unknown exception caught\n";
  }
}
